

#ifndef CMDLINE_H
#define CMDLINE_H

#include <string>
#include <vector>
#include "param.h"

// A class for care and feeding of command line parameters
// if things go well it will construct a set of parameteres and
// feed the command line arguments to the parameter set.  
// Otherwise some diagnostic will be printed on the standard error.
// each parameter gets:
//     a description of the parameter for use in diagnostics
//     a flag character (should be a string?)
//     a pointer to the target data item
// an iterated patameter gets a pointer to an integer that will record
//     the number of instances of the command line argument at run time

class cmdLine
{
private: 
    vector<param*> m_parameter;
    int m_numParameters;      // how many there are
    string m_description;     // to print meaning of the program
    string m_programName;     // name used to call program
    int m_numNonFlagArgs;    // we can only have one

public:
    cmdLine(string programName, string description);
    ~cmdLine();

             ////////////////////////////////////////////////////
             // methods to add various types of parameter      //
             ////////////////////////////////////////////////////
    
    // add an integer parameter
    void addParam(const char flag, int  *target, const string description, 
            const int options=0, 
            int *numExamples=NULL, const int maxExamples=1);
    
    // add a float parameter
    void addParam(const char flag, float  *target, const string description, 
            const int options=0, 
            int *numExamples=NULL, const int maxExamples=1);

    // add a string parameter
    void addParam(const char flag, string *target, const string description, 
            const int options=0, 
            int *numExamples=NULL, const int MaxExamples=1);

    // add a switch parameter
    void addParam(const char flag, bool *target, const string description, 
            const int options=0, 
            int *numExamples=NULL, const int MaxExamples=1);

    // add a compound parameter
    void addParam(const char flag, const string description, 
            const int options=0, 
            int *numExamples=NULL, const int MaxExamples=1);

    // ensure that whether we are iterated aggrees with the patameter
    void checkAllOrNoneIterated(bool iterated);

    // add an int component to a compound parameter
    void addToCompound(int *target, const string description,
            const int options=0, 
            int *numExamples=NULL, const int MaxExamples=1);

    // add a float component to a compound parameter
    void addToCompound(float *target, const string description,
            const int options=0, 
            int *numExamples=NULL, const int MaxExamples=1);

    // add a string component to a compound parameter
    void addToCompound(string *target, const string description,
            const int options=0, 
            int *numExamples=NULL, const int MaxExamples=1);

    // dont't add a switch component to a compound parameter
    void addToCompound(bool *target, const string description,
            const int options=0, 
            int *numExamples=NULL, const int MaxExamples=1);

             ////////////////////////////////////////////////////////
             // The method that fetches data from the command line //
             ////////////////////////////////////////////////////////
    // fetch the parameter values out of the argv list.  This method must
    // be called after the parameters have been constructed.
    void getArgs(const int argc, char *argv[]);

private:
    // is this command line argument asking for a synopsis report?
    bool isSynopsis(const char *flag) const
        {return ((flag[0] == '-') && (tolower(flag[1]) == 'h'));};

    // is this command line argument asking for synopsis and descriptions?
    bool isHelp(const char *flag){return (strcmp(flag,"-H")==false);};

	// has this flag been used before? This method exits if newFlag is reused.
	void checkIfFlagReused(char newFlag);

    // print descriptions of the command line parameters
    void printDescription() const;

    // print a usage line
    void usage() const;

    // print defaults for command line arguments
    void printDefault() const;
};
#endif
