
#include "compoundParam.h"
#include <cctype>

compoundParam::compoundParam(const char flag, const string description, 
        const int options, int *numExamples, const int maxExamples)
    : param(flag, description, options, numExamples, maxExamples)
{
    initParam();
} 

compoundParam::~compoundParam()
{
    for (int i=0; i<m_numComponents; i++)
    {
        delete m_component[i];
    }
}

void
compoundParam::addComponent(param *component, const int maxExamples=1)
{
    m_component.push_back(component);
    m_numComponents++;
    if (m_maxExamples > maxExamples)
        m_maxExamples = maxExamples;
}

void 
compoundParam::initParam()
{
    m_maxExamples = 1000;
    m_typeName = "compound";
    m_numComponents = 0;
}


void 
compoundParam::printType() const
{
    for (int i=0; i<m_numComponents; i++)
    {
        m_component[i]->printType();
        cerr << " " ;
    }
}

void
compoundParam::describeParam() const
{
    cerr << m_description;
    cerr << " " ;
    for (int i=0; i<m_numComponents; i++)
    {
        cerr << "<" ;
        m_component[i]->describeParam();
        cerr << "> " ;
    }
}

void 
compoundParam::printValue() const
{
    for (int i=0; i<m_numComponents; i++)
    {
        m_component[i]->printValue();
        cerr << " " ;
    }
}

void
compoundParam::setParam(char **arg)
{
    wrongNumComponents();
    (*m_numExamples)++;
    for (int i=0; i<m_numComponents; i++)
    {
        m_component[i]->setParam(arg + i);
    }
}

bool
compoundParam::wrongNumComponents() const
{
    if (m_numComponents < 1)
    {
        paramError("you need at least one component in a compound parameter.");
        return  true;
    }
    else
    {
        return  false;
    }
}

