

#include <string>
#include <iostream>
#include <iomanip>

#include "cmdLine.h"

// todo: implement compound, bool, float


int main(int argc, char *argv[])
{
    // declare and initialise the arglist
    cmdLine cmd(argv[0], "test command line argument processing");


    // simple parameters.  The type of parameter is determined by the
    // type of the variable being used.  We can have:
    int angels = 42;                  // integer parameters,
    cmd.addParam('a', &angels, "Number of angels on the pin");

    float pi = 3.14159;               // float parameters,
    cmd.addParam('a', &pi, "current value of PI");

    string godName = "God";           // string parameters,
    cmd.addParam('d', &godName, "The current name of the major deity");

    bool godExists = true;            // boolean parameters,
    cmd.addParam('g', &godExists, "does god exist?");

    // any parameter can be REQUIRED, ITERATED or NONFLAG or any
    // combination of these.   (Except that a switch can't be REQUIRED.)
    
    // a REQUIRED parameter.  If it does not occur at runtime, the
    // program will abort with a diagnostic
    int beastNumber = 0;
    cmd.addParam('b', &beastNumber, "number of the beast", REQUIRED);
    
    // a NONFLAG string parameter.  "-f" is optional on cammand line
    // string outputFile = "a.out"
    // cmd.addParam('f', &outputFile, "the output file name", NONFLAG);
    // This example is commented out because more than one NONFLAG
    // parameter would be ambiguous at runtime (and not alowed by
    // getargs()) and I want to show an ITERATED NONFLAG example.

    // slightly more complicated, an iterated string paramter.  The number of
    // '-n' arguments found at runtime will be stored in numNames.  More than
    // 3 at runtime will produce a diagnostic.  By OR'ing in the NONFLAG, we
    // make this parameter also a nonflag  parameter as above.  
    // Notice, we pass the address of the first element of the array.
    const int MAX_NAMES = 3;
    string nameList[MAX_NAMES];
    int numNames = 0;
    cmd.addParam('n', nameList, "Important names", ITERATED, 
            &numNames, MAX_NAMES);

    // if you want a compound parmeter, you first create the compound,
    // then add its components

    // Here is a collection of positioned labels to specify from the command
    // line as compounds.  I recommend using indentation to show the
    // structure of the compound argument.  If you use braces, it will affect
    // the scope of variables you declare inside.
    // This is probably not the way to do it.  The compound parameter
    // should probably feed information into an object, but I can't see
    // how to do this without the cmdLine object being compiled with knowledge
    // about the target object.  That would involve modifying and recompiling
    // getargs.  That would be bad.
    // The programmer should not have to use mutiple variables that will
    // always have the same value.  It is done this way because the
    // arrays of targets are independant arrays rather than structures.
	//
	// It is a NONFLAG parameter
	
    const int MAX_LABELS = 3;
    int numLabels = 0;        // create the compound
    cmd.addParam('L', "box info", ITERATED|NONFLAG, &numLabels, MAX_LABELS);
        // add a component for the label's X position
        int x[MAX_LABELS];
        int numXords = 0;
        cmd.addToCompound(x, "label X position", ITERATED, &numXords, 
                MAX_LABELS); 

        // add a component for the label's Y position
        int y[MAX_LABELS];
        int numYords = 0;
        cmd.addToCompound(y, "label Y position", ITERATED, &numYords, 
                MAX_LABELS); 

        // add a component for the label
        string labels[MAX_LABELS];
        int numStrings = 0;
        cmd.addToCompound(labels, "label string", ITERATED, &numStrings, 
                MAX_LABELS); 


    cout << endl << "before getargs()...." << endl;
    cout << "angels = " << angels << endl;
    cout << "pi = " << pi << endl;
    cout << "godName = " << godName << endl;
    cout << "godExists = " << godExists << endl;
    cout << "positioned labels" << endl;
    for (int i=0; i< numLabels; i++)
    {
        cout << i << " (" << x[i] << "," << y[i] << ") " ;
        cout  << labels[i] << endl;
    }

	// parse the runtime argument list
    cmd.getArgs(argc, argv);

    cout << endl << "after getargs()...." << endl;
    cout << "angels = " << angels << endl;
    cout << "pi = " << pi << endl;
    cout << "godName = " << godName << endl;
    cout << "godExists = " << godExists << endl;


    cout << "positioned labels" << endl;
    for (int i=0; i< numLabels; i++)
    {
        cout << i << " (" << x[i] << "," << y[i] << ") " ;
        cout  << labels[i] << endl;
    }
}
