

#ifndef PARAM_H
#define PARAM_H

#include <string>

#define REQUIRED 1
#define ITERATED 2
#define NONFLAG  4

// make virtual:
// 1. produce a virtual method that does only what is unique to the sub-classes
//         it should be some kind of set or get method
// 2. call the new virtual method from existing the over-ridden methods
//         the over-ridden methods should become identical as a result
// 3. move over-ridden method into base and delete from sub-classes
//
//
//  abstract base class for paramerter managers
//
class param{
protected: 
    char m_flag;         // the command line flag
    bool m_iterated;     // option  (an iterated parameter)
    bool m_nonFlag;      // option  (a non-flag paramter)
    bool m_required;     // option  (a manditory parameter)
    string m_description;// an online help description
    string m_typeName;   // a string to represent the type.

    bool m_used;         // has this parameter been used?
    int *m_numExamples;
        // value is 0 for a flag, 1 for a normal param, N for an iterated param
        // meaning is "number of instances of argument on command line at 
        // run time"
    int m_localNumExamples;  // a local version in for non-iterated parameters
    int m_maxExamples;  // for interated parameters where the number is limited
	                    // by the size of a target array

    int m_numComponents;
        //value is 0 for switches, 1 for non-compound, >=1 for compound
        //meaning is "number of components in parameter"

    void init(int options); // set up the option switches

public:
    param();
    param(const char flag, const string description, const int options);
    param(const char flag, const string description, const int options, int *numExamples, const int MaxExamples);
    virtual ~param();
	//~param() { m_numInstances--; cerr <<m_numInstances << endl; } ;

    // use the argument if we can and return number of arguments actually used
    int useArgs(const int param, const int argc, char *argv[]);
    bool checkIsUnusedRequired() const;
    virtual bool checkWrongNumComponents() const {return false;};
    bool isIterated() const {return m_iterated;};
    bool flagReused(const char newFlag) const {return (newFlag == m_flag);};
	// how to use this parameter
    void usage() const;
	// print description of this parameter
    void printDescription() const;
    virtual void describeParam() const {cerr << m_description;};
    
	// print the cmd line arguments that would set this parameter to its
	// default
    void printDefault() const;
	void paramError(string message) const;

	virtual void addComponent(param *component,  const int MaxExamples=1);
    virtual void setParam(char **arg)=0;

    virtual void printValue() const  =0 ;
    virtual void printType() const {cerr << m_typeName;} ;
private:
    static int m_numClassInstances;  // automatically set to zero
	static int m_numNonFlagArgs;    // we can only have one non flag argument
    bool canUseFlaggedArg(const int arg, const int argc, char *argv[]) const;
    bool canUseNonFlagArg(const int arg, const int argc, char *argv[]) const;
};
#endif
